freeslot("mt_hitbox","spr_htbx","s_hitbox")
mobjinfo[MT_HITBOX] = {
        spawnstate = S_HITBOX,
        flags = MF_SCENERY|MF_NOTHINK|MF_NOBLOCKMAP
}

states[S_HITBOX] = {
        sprite = SPR_HTBX,
        frame = A,
		tics = 1,
        nextstate = S_HITBOX
}

local CV_Rings = CV_RegisterVar{
	name = "hitbox_rings",
	defaultvalue = 1,
	flags = CV_NETVAR,
	PossibleValue = {MIN = 0, MAX = 1}
}

local CV_Player = CV_RegisterVar{
	name = "hitbox_player",
	defaultvalue = 1,
	flags = CV_NETVAR,
	PossibleValue = {MIN = 0, MAX = 1}
}

local CV_Color = CV_RegisterVar{
	name = "hitbox_color",
	defaultvalue = 1,
	flags = CV_NETVAR,
	PossibleValue = {MIN = 0, MAX = 1}
}

local CV_Distance = CV_RegisterVar{
	name = "hitbox_distance",
	defaultvalue = 512,
	flags = CV_NETVAR,
	PossibleValue = {MIN = 0, MAX = 2560}
}

local CV_Faces = CV_RegisterVar{
	name = "hitbox_faces",
	defaultvalue = 1,
	flags = CV_NETVAR,
	PossibleValue = CV_OnOff
}

local Hitboxes = {}
local Vertices = {}
local Faces = {}


local CanSeeHitbox = function(mo)
	local dist = R_PointToDist(mo.x,mo.y)
-- 	if not P_CheckSight(displayplayer.realmo, mo)
	if dist > CV_Distance.value*FRACUNIT
		return false
	end
	if mo.type == MT_PLAYER and dist < FRACUNIT*72 then return false end
	return true
end

local DoFace = function(mo)
	//Object does not exist anymore
	if not(mo and mo.valid) then return nil end
	//Target does not exist anymore
	local target = mo.target
	if not(target and target.valid) then
		P_RemoveMobj(mo)
	return nil end
	//Debug hitboxes are turned off
	if CV_Faces.value == 0
		P_RemoveMobj(mo)
	return nil end
	//Ring hitboxes are turned off
	if target.type == MT_RING and not(CV_Rings.value) then
		P_RemoveMobj(mo)
	return nil end
	//Player hitboxes are turned off
	if target.type == MT_PLAYER and not(CV_Player.value) then
		P_RemoveMobj(mo)
	return nil end
	//Hitbox is too far away
	if not(CanSeeHitbox(target)) then
		P_RemoveMobj(mo)
	return nil end
	//All checks passed. Updating position
	P_TeleportMove(mo,target.x+FixedMul(mo.xo,target.scale),target.y+FixedMul(mo.yo,target.scale),target.z+FixedMul(mo.zo,target.scale))
	target.vertices_exist = true //Confirming to our target object that vertices still exist

	//Do color coding
	if not(CV_Color.value) then
		mo.color = SKINCOLOR_JET
	elseif target.player then
		mo.color = SKINCOLOR_BLUE
	elseif target.flags&(MF_PAIN|MF_MISSILE) then
		mo.color = SKINCOLOR_RED
	elseif target.flags&MF_SOLID then
		mo.color = SKINCOLOR_GREY
	elseif target.flags&MF_PUSHABLE then
		mo.color = SKINCOLOR_PURPLE
	elseif target.flags&(MF_ENEMY|MF_BOSS) then
		mo.color = SKINCOLOR_ORANGE
	elseif target.flags&(MF_SPECIAL|MF_SPRING) then
		mo.color = SKINCOLOR_YELLOW
	else
		mo.color = SKINCOLOR_GREEN
	end
	return mo
end

local trans = FF_TRANS70
local MakeBox = function(mo)
	if not(mo and mo.valid) return end
	local face
	for n = 1, 4 do
		local angle = FixedAngle((90*n)<<FRACBITS)
		local x = FixedMul(cos(angle),mo.radius)
		local y = FixedMul(sin(angle),mo.radius)
		face = P_SpawnMobjFromMobj(mo, x, y, 0, MT_HITBOX)
		face.xo = x
		face.yo = y
		face.zo = 0
		face.angle = angle+ANGLE_90
		face.renderflags = $|RF_PAPERSPRITE
		face.frame = 1|trans
		face.spritexscale = mo.radius
		face.spriteyscale = mo.height>>1
-- 		face.flags2 = $|MF2_SHADOW
		face.target = mo
		table.insert(Faces,face)
		DoFace(face)
	end
	for n = 1, 2 do
		local z = mo.height * (n&1)
		face = P_SpawnMobjFromMobj(mo, 0, 0, z, MT_HITBOX)
		face.renderflags = $|(RF_FLOORSPRITE|RF_NOSPLATBILLBOARD)
		face.xo = 0
		face.yo = 0
		face.zo = z
		face.frame = 2|trans
		face.spritexscale = mo.radius
		face.spriteyscale = mo.radius
-- 		face.flags2 = $|MF2_SHADOW
		face.target = mo
		table.insert(Faces,face)
		DoFace(face)
	end
end

local UpdateHitbox = function(mo)
	if not(mo and mo.valid) then return nil end
	if mo.type == MT_RING and not(CV_Rings.value) then return true end
	if mo.vertices_exist then
		mo.vertices_exist = false //Reset the check for next frame
	return true end
	if not(CanSeeHitbox(mo)) then return true end
	local r = mo.radius
	local h = mo.height
	if CV_Faces.value
		MakeBox(mo)
	end	
	return true
end

local flags = MF_SPECIAL|MF_MISSILE|MF_PAIN|MF_ENEMY|MF_BOSS|MF_SOLID|MF_PUSHABLE|MF_SPRING

addHook("MobjSpawn",function(mo)
		if not(mo.flags&flags or mo.type == MT_PLAYER)
		then return end
		Hitboxes[#Hitboxes+1] = mo
		mo.vertices_exist = false
end,MT_NULL)

addHook("PostThinkFrame",function()
	

-- 	//Update faces
	local queue = Faces
	Faces = {}
	local size = #queue
	for n = 1, size do
		local face = queue[n]
		if DoFace(face) then
			Faces[#Faces+1] = face
		end
	end
		
	//Update hitboxes
	queue = Hitboxes
	Hitboxes = {}
	local size = #queue
	for n = 1, size do
		local hitbox = queue[n]
		if UpdateHitbox(hitbox) then
			Hitboxes[#Hitboxes+1] = hitbox
		end
	end
end)

addHook("MapChange",function()
	Vertices = {}
	Hitboxes = {}
	Faces = {}
end)

